@@ -1,4 +1,4 @@
-#' @title Methods for Calculating Diagnostics
+#' @title Methods for Calculating Margins
 #' @name Margins-Methods
 #' @docType methods
 #'
@@ -6,7 +6,9 @@
 #' calcMargins,ANY-method
 #' calcMargins,AIDS-method
 #' calcMargins,Bertrand-method
+#' calcMargins,LogitCournot-method
 #' calcMargins,VertBargBertLogit-method
+#' calcMargins,VertBargBertLogitNests-method
 #' calcMargins,LogitCap-method
 #' calcMargins,Auction2ndLogit-method
 #' calcMargins,Auction2ndLogitNests-method
@@ -44,6 +46,7 @@ setMethod(
   signature= "Bertrand",
   definition=function(object,preMerger=TRUE, level=FALSE){
 
+    output <- ifelse(object@output,-1,1)
 
 
     if( preMerger) {
@@ -63,8 +66,8 @@ setMethod(
     
     elast <-  elast(object,preMerger)
     
-    margins <-  try(-1 * as.vector(solve(t(elast)*owner) %*% (revenue * diag(owner))) / revenue,silent=TRUE)
-    if(any(class(margins) == "try-error")){margins <- -1 * as.vector(MASS::ginv(t(elast)*owner) %*% (revenue * diag(owner))) / revenue}
+    margins <-  try(output * as.vector(solve(t(elast)*owner) %*% (revenue * diag(owner))) / revenue,silent=TRUE)
+    if(any(class(margins) == "try-error")){margins <- output * as.vector(MASS::ginv(t(elast)*owner) %*% (revenue * diag(owner))) / revenue}
     
     
     
@@ -78,6 +81,56 @@ setMethod(
 )
 
 
+## compute margins
+#'@rdname Margins-Methods
+#'@export
+setMethod(
+  f= "calcMargins",
+  signature= "LogitCournot",
+  definition=function(object,preMerger=TRUE, level=FALSE){
+    
+    
+    output <- ifelse(object@output,-1,1)
+    alpha <- object@slopes$alpha
+    idx   <-  object@normIndex
+    
+    if( preMerger) {
+      
+      prices <- object@pricePre
+      owner  <- object@ownerPre
+      
+    }
+    
+    else{
+      prices <- object@pricePost
+      owner  <- object@ownerPost
+      
+    }
+    
+    shares <- calcShares(object,preMerger=preMerger,revenue=FALSE)
+    
+    if(is.na(idx)){
+      idxShare <- 1 - sum(shares)
+    }
+    else{
+      idxShare <- shares[idx]
+     
+    }
+    sharesFirm <- as.numeric(owner %*% shares) 
+    
+    
+    margins <- output*(1 + sharesFirm/idxShare)/alpha
+    
+    if(!level) {margins <- margins/prices }
+    
+    names(margins) <- object@labels
+    
+    return(as.vector(margins))
+  }
+  
+)
+
+
 ## compute margins
 #'@rdname Margins-Methods
 #'@export
@@ -86,6 +139,8 @@ setMethod(
   signature= "Bargaining2ndLogit",
   definition=function(object,preMerger=TRUE,exAnte=FALSE,level=TRUE){
   
+    
+    
     if( preMerger) {
       
       barg <- object@bargpowerPre
@@ -111,6 +166,7 @@ setMethod(
   signature= "BargainingLogit",
   definition=function(object,preMerger=TRUE, level=FALSE){
     
+    output <- ifelse(object@output,-1,1)
     
     alpha <- object@slopes$alpha
     
@@ -146,9 +202,10 @@ setMethod(
     diag(margins) <- diag(owner) +  diag(margins)
     margins <- solve(t(margins))
     
-    margins <-  as.vector(margins %*% ((log(1-shares)*diag(owner))/(alpha*(barg*div - 
+    margins <-  as.vector(margins %*% ((log(1-shares)*diag(owner))/(-1*output*alpha*(barg*div - 
                                                                log(1-shares)))))
     
+    margins <- margins
     
     if(!level) {margins <- margins / prices }
     
@@ -173,6 +230,15 @@ setMethod(
     up <- object@up
     down <- object@down
     alpha <- down@slopes$alpha
+    
+    chain_level <- object@chain_level
+    
+    ## Set downstream margin equal to pre-merger value when chain_level=="wholesaler
+    preMerger_chain <- preMerger
+    if(chain_level=="wholesaler") {
+      preMerger_chain <- TRUE
+       if(!preMerger) down@ownerPre <- ownerToMatrix(down,preMerger= TRUE)
+    }
   
     #is2nd <- grepl("2nd",class(object))
   
@@ -231,13 +297,12 @@ setMethod(
     elast.inv <- try(solve(ownerDown * elast),silent=TRUE)
     if(any(class(elast.inv) == "try-error")){elast.inv <- MASS::ginv(ownerDown * elast)}
     
-    if(!is2nd) {marginsDown <- calcMargins(down, preMerger=preMerger,level=TRUE)}
-    else{
+    if(is2nd){
       alpha <- down@slopes$alpha
       down@slopes$meanval <- down@slopes$meanval + alpha *(priceUp - down@priceOutside) 
-      marginsDown <- calcMargins(down, preMerger=preMerger,level=TRUE)
     }
     
+    marginsDown <- calcMargins(down, preMerger=preMerger_chain,level=TRUE)
     
     div <- tcrossprod(1/(1-shareDown),shareDown)*shareDown
     diag(div) <- -shareDown
@@ -248,6 +313,11 @@ setMethod(
     
     #marginsUp <-  as.vector(solve(ownerUpLambda * div) %*% (((ownerDownLambda * div) %*% (marginsDown)))) 
     
+    if(chain_level == "retailer" && !preMerger){ 
+      #marginsUp <- calcMargins(object,preMerger=TRUE,level=TRUE)$up
+      marginsUp <- up@pricePre - up@mcPost
+      }
+    else{
     marginsUpPart <-  try(solve(ownerUpLambda * div) %*% (ownerDownLambda * div) ,silent=TRUE)
     if(any(class(marginsUpPart) == "try-error")){
       marginsUpPart <-  MASS::ginv(ownerUpLambda * div) %*% (ownerDownLambda * div) 
@@ -260,8 +330,17 @@ setMethod(
     }
     marginsUp <- drop(marginsUp %*% marginsUpPart %*% marginsDown)
     
+    }
 
-    if(!is2nd){marginsDown <-  marginsDown - elast.inv %*% ( (ownerVDown * elast) %*% marginsUp )}
+    if(!is2nd){
+      
+      if(chain_level == "wholesaler" && !preMerger){ 
+        #marginsDown <- calcMargins(object,preMerger=TRUE,level=TRUE)$down
+        marginsDown <- down@pricePre - down@mcPost - priceUp
+        } 
+      else{ marginsDown <-  marginsDown - elast.inv %*% ( (ownerVDown * elast) %*% marginsUp )}
+      
+      }
   
     if(!level) {
       marginsUp <- marginsUp/priceUp
@@ -280,6 +359,184 @@ setMethod(
   
 )
 
+#'@rdname Margins-Methods
+#'@export
+setMethod(
+  f= "calcMargins",
+  signature= "VertBargBertLogitNests",
+  definition=function(object,preMerger=TRUE, level=FALSE){
+
+    #check if class is 2nd score Logit
+    is2nd <- any(grepl("2nd",class(object)))
+
+    up <- object@up
+    down <- object@down
+    alpha <- down@slopes$alpha
+    nestParm <- 1 - down@slopes$sigma #adjust to accomodate below different nesting paramter definition
+    
+    nests <- down@nests
+    nests <- droplevels(nests)
+    nestMat <- tcrossprod(model.matrix(~-1+nests))
+
+    
+    chain_level <- object@chain_level
+
+    ## Set downstream margin equal to pre-merger value when chain_level=="wholesaler
+    preMerger_chain <- preMerger
+    if(chain_level=="wholesaler") {
+      preMerger_chain <- TRUE
+      if(!preMerger) down@ownerPre <- ownerToMatrix(down,preMerger= TRUE)
+    }
+
+    #is2nd <- grepl("2nd",class(object))
+
+    ownerUp <- ownerToMatrix(up,preMerger = preMerger)
+    ownerDown <- ownerToMatrix(down,preMerger= preMerger)
+
+    nprods <- nrow(ownerDown)
+
+    if( preMerger) {
+      bargparm <- up@bargpowerPre
+
+      #if(length(up@pricePre) == 0 ){
+      #priceUp <- up@prices
+      #}
+
+      #else{
+      priceUp <- up@pricePre
+      # }
+      #if(length(down@pricePre) == 0 ){
+      # priceDown <- down@prices
+      #  down@pricePre <- priceDown
+      #}
+      #else{
+      priceDown <- down@pricePre
+
+      # }
+
+      ownerDownLambda <- object@ownerDownLambdaPre
+      ownerUpLambda <- object@ownerUpLambdaPre
+      ownerVDown <- object@ownerDownPre
+
+
+      #down@mcPre <- down@mcPre + priceUp
+      down@ownerPre <- ownerDown
+
+    }
+
+    else{
+      bargparm <- up@bargpowerPost
+
+
+      priceUp <- up@pricePost
+      priceDown <- down@pricePost
+
+      ownerDownLambda <- object@ownerDownLambdaPost
+      ownerUpLambda <- object@ownerUpLambdaPost
+      ownerVDown <- object@ownerDownPost
+
+      down@ownerPost <- ownerDown
+
+    }
+
+    shareDown <- calcShares(object,preMerger=preMerger, revenue=FALSE)
+    shareBetweenDown <- as.vector(unname(tapply(shareDown,nests,sum)[nests]))
+    shareWithinDown <- shareDown/shareBetweenDown
+
+    #elast <-  -alpha*tcrossprod(shareDown)
+    #diag(elast) <- alpha*shareDown + diag(elast)
+
+
+    elastDiff <- -alpha*tcrossprod(shareDown)
+    elastSame <- -alpha*tcrossprod(shareDown*(1 + nestParm[nests] /( 1-nestParm[nests] )*(1/shareBetweenDown)),shareDown)
+    elast <- elastSame*nestMat + elastDiff*(1-nestMat)
+    diag(elast) <- alpha/(1-nestParm[nests])*shareDown*(1-nestParm[nests]*shareWithinDown - (1-nestParm[nests])*shareDown)
+    elast.inv <- try(solve(ownerDown * elast),silent=TRUE)
+    if(any(class(elast.inv) == "try-error")){elast.inv <- MASS::ginv(ownerDown * elast)}
+
+    marginsDown = - as.vector(elast.inv %*% shareDown)
+
+
+
+
+    # if(is2nd){
+    #   alpha <- down@slopes$alpha
+    #   down@slopes$meanval <- down@slopes$meanval + alpha *(priceUp - down@priceOutside)
+    # }
+
+    #marginsDown <- calcMargins(down, preMerger=preMerger_chain,level=TRUE)
+
+    # div <- tcrossprod(1/(1-shareDown),shareDown)*shareDown
+    # diag(div) <- -shareDown
+    # div <- as.vector(div)
+
+
+
+    divSameNest <- (1-shareWithinDown)^nestParm[nests]*(1 - shareBetweenDown + shareBetweenDown * (1- shareWithinDown)^(1-nestParm[nests]) )
+    divSameNest <-  tcrossprod((1/divSameNest - 1), shareDown)
+    ##    matrix(shares * (1/divSameNest - 1), nrow = nprods,ncol=nprods,byrow=TRUE)
+    divDiffNest <- shareBetweenDown * (1 - (1- shareWithinDown)^(1-nestParm[nests]) )
+    divDiffNest <-  1/divDiffNest - 1
+    divDiffNest <-  tcrossprod(1/divDiffNest, shareDown)
+    ##   matrix(shares/divDiffNest, nrow=nprods, ncol=nprods, byrow=TRUE)
+
+    div <- divSameNest * nestMat + divDiffNest*(1-nestMat)
+
+    diag(div) <- -shareDown
+    div[is.na(div)] <- 0
+
+
+
+    #marginsDown <-  marginsDown - elast.inv %*% ( (ownerVDown * elast) %*% (priceCandUp-mcUp) )
+
+    #marginsUp <-  as.vector(solve(ownerUpLambda * div) %*% (((ownerDownLambda * div) %*% (marginsDown))))
+
+    if(chain_level == "retailer" && !preMerger){
+      #marginsUp <- calcMargins(object,preMerger=TRUE,level=TRUE)$up
+      marginsUp <- up@pricePre - up@mcPost
+    }
+    else{
+      marginsUpPart <-  try(solve(ownerUpLambda * div) %*% (ownerDownLambda * div) ,silent=TRUE)
+      if(any(class(marginsUpPart) == "try-error")){
+        marginsUpPart <-  MASS::ginv(ownerUpLambda * div) %*% (ownerDownLambda * div)
+      }
+
+      marginsUp <- try(solve(diag(nprods) + (marginsUpPart %*% elast.inv %*%  (ownerVDown * elast))),silent=TRUE)
+
+      if(any(class(marginsUp) == "try-error")){
+        marginsUp <- MASS::ginv(diag(nprods) + (marginsUpPart %*% elast.inv %*%  (ownerVDown * elast)))
+      }
+      marginsUp <- drop(marginsUp %*% marginsUpPart %*% marginsDown)
+
+    }
+
+    if(!is2nd){
+
+      if(chain_level == "wholesaler" && !preMerger){
+        #marginsDown <- calcMargins(object,preMerger=TRUE,level=TRUE)$down
+        marginsDown <- down@pricePre - down@mcPost - priceUp
+      }
+      else{ marginsDown <-  marginsDown - elast.inv %*% ( (ownerVDown * elast) %*% marginsUp )}
+
+    }
+
+    if(!level) {
+      marginsUp <- marginsUp/priceUp
+      marginsDown <- marginsDown/priceDown
+    }
+
+
+
+
+    names(marginsUp) <- up@labels
+    names(marginsDown) <- down@labels
+
+    return(list(up=as.vector(marginsUp), down = as.vector(marginsDown))
+    )
+  }
+
+)
+
 
 
 #'@rdname Margins-Methods
@@ -305,6 +562,8 @@ setMethod(
   definition=function(object,preMerger=TRUE, level=FALSE){
 
 
+     output <- object@output
+     
     if(preMerger){
       prices <- object@pricePre
     }
@@ -315,7 +574,8 @@ setMethod(
 
 
 
-    margin <- prices - mc
+    if(output){margin <- prices - mc}
+    else{margin <- mc - prices}
     if(!level){margin <- margin/prices}
 
     dimnames(margin) <- object@labels
@@ -401,6 +661,8 @@ setMethod(
 
 )
 
+
+
 ## compute margins
 #'@rdname Margins-Methods
 #'@export
@@ -410,6 +672,8 @@ setMethod(
   definition=function(object,preMerger=TRUE,exAnte=FALSE,level=TRUE){
 
 
+    output <- ifelse(object@output,1,-1)
+    
     nprods <- length(object@shares)
    
    
@@ -431,7 +695,7 @@ setMethod(
     shares <- calcShares(object,preMerger=preMerger,revenue=FALSE)
     shares <- shares[subset]
     firmShares <- drop(owner %*% shares)
-    margins[subset] <-  log(1-firmShares)/(alpha * firmShares)
+    margins[subset] <-  output*log(1-firmShares)/(alpha * firmShares)
 
     if(exAnte){ margins[subset] <-  margins[subset] * shares}
     if(!level){margins <- margins/prices}
@@ -450,7 +714,7 @@ setMethod(
   signature= "Auction2ndLogitNests",
   definition=function(object,preMerger=TRUE,exAnte=FALSE,level=FALSE){
     
-    
+    output <- ifelse(object@output,1,-1)
     nprods <- length(object@shares)
     
     
@@ -491,7 +755,7 @@ setMethod(
     ownerValue <-   1 - (1 - ((owner*nestMat)%*%sharesIn))^sigma[nests]
     ownerValue <-  drop(1 - owner %*%(ownerValue * sharesAcross/dupCnt))
     
-    margins[subset] <-  log(ownerValue)/(alpha*firmShares) 
+    margins[subset] <-  output*log(ownerValue)/(alpha*firmShares) 
     
     
     if(exAnte){ margins[subset] <-  margins[subset] * shares}
